"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("@typescript-eslint/utils");
var utils_2 = require("../utils");
var reactLifecycleMethods = [
    'UNSAFE_componentWillMount',
    'componentWillMount',
];
exports.default = (0, utils_2.createRule)({
    name: 'no-state-access',
    meta: {
        docs: {
            description: "Disallow state accsess in ReSub 'componentWillMount'",
            recommended: 'error',
        },
        messages: {
            stateAccsessError: "'this.state' is undefined in 'componentWillMount' callback",
        },
        schema: [],
        type: 'suggestion',
    },
    defaultOptions: [],
    create: function (context) {
        var isState = function (name) { return name === 'state'; };
        var stack = [];
        var enterMethod = function (node) {
            var isComponentWillMount = node.key.type === utils_1.AST_NODE_TYPES.Identifier &&
                reactLifecycleMethods.includes(node.key.name) &&
                (0, utils_2.isReSubComponent)(context.getScope());
            stack.push(isComponentWillMount);
        };
        var exitMethod = function () {
            stack.pop();
        };
        var getStateNode = function (node) {
            var VariableDeclarator = utils_1.AST_NODE_TYPES.VariableDeclarator, MemberExpression = utils_1.AST_NODE_TYPES.MemberExpression, ObjectPattern = utils_1.AST_NODE_TYPES.ObjectPattern, Identifier = utils_1.AST_NODE_TYPES.Identifier, Property = utils_1.AST_NODE_TYPES.Property;
            if (node.type === MemberExpression &&
                node.property.type === Identifier &&
                isState(node.property.name)) {
                return node.property;
            }
            if (node.type === VariableDeclarator &&
                node.id &&
                node.id.type === ObjectPattern) {
                var property = node.id.properties.find(function (property) {
                    return property.type === Property &&
                        property.key.type === Identifier &&
                        isState(property.key.name);
                });
                if ((property === null || property === void 0 ? void 0 : property.type) === Property && property.key.type === Identifier) {
                    return property.key;
                }
            }
            return undefined;
        };
        var validate = function (node) {
            var isComponentWillMount = stack.length && stack[stack.length - 1];
            if (!isComponentWillMount) {
                return;
            }
            if (node.parent) {
                var stateNode = getStateNode(node.parent);
                if (stateNode) {
                    context.report({ messageId: 'stateAccsessError', node: stateNode });
                }
            }
        };
        return {
            MethodDefinition: enterMethod,
            'MethodDefinition:exit': exitMethod,
            ThisExpression: validate,
        };
    },
});
