"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("@typescript-eslint/utils");
var utils_2 = require("../utils");
var reactLifecycleMethods = [
    'UNSAFE_componentWillReceiveProps',
    'UNSAFE_componentWillUpdate',
    'UNSAFE_componentWillMount',
    'componentWillReceiveProps',
    'componentWillUnmount',
    'componentWillUpdate',
    'componentWillMount',
    'componentDidUpdate',
    'componentDidMount',
    '_buildInitialState',
];
exports.default = (0, utils_2.createRule)({
    name: 'override-calls-super',
    meta: {
        docs: {
            description: "Require 'super' calls in overridden methods",
            recommended: 'error',
        },
        messages: {
            callSuperError: "Method override must call 'super.{{methodName}}'",
            superShouldBeFirstStatement: "Method override must call 'super.{{methodName}}' in the top-level statements of the method body",
        },
        schema: [],
        type: 'suggestion',
    },
    defaultOptions: [],
    create: function (context) {
        var isSuperCall = function (statement) {
            return (statement === null || statement === void 0 ? void 0 : statement.type) === utils_1.AST_NODE_TYPES.ExpressionStatement &&
                statement.expression.type === utils_1.AST_NODE_TYPES.CallExpression &&
                statement.expression.callee.type === utils_1.AST_NODE_TYPES.MemberExpression &&
                statement.expression.callee.object.type === utils_1.AST_NODE_TYPES.Super;
        };
        var validate = function (node) {
            if (node.key.type !== utils_1.AST_NODE_TYPES.Identifier ||
                node.value.type !== utils_1.AST_NODE_TYPES.FunctionExpression ||
                !(node.value.body &&
                    node.value.body.type === utils_1.AST_NODE_TYPES.BlockStatement)) {
                return;
            }
            if (!(node.static && node.key.name === 'getDerivedStateFromProps') &&
                !reactLifecycleMethods.includes(node.key.name)) {
                return;
            }
            if (!(0, utils_2.isReSubComponent)(context.getScope())) {
                return;
            }
            var body = node.value.body.body;
            var firstStatement = body[0];
            if (!isSuperCall(firstStatement)) {
                context.report({
                    messageId: body.some(isSuperCall)
                        ? 'superShouldBeFirstStatement'
                        : 'callSuperError',
                    node: node,
                    data: { methodName: node.key.name },
                });
            }
        };
        return {
            MethodDefinition: validate,
        };
    },
});
